"""
https://github.github.com/gfm/#backslash-escapes
"""
import pytest

from .utils import act_and_assert


@pytest.mark.gfm
def test_backslash_escapes_extra_1():
    """
    Test case backslash extra 1:  backslash before the code span open
    """

    # Arrange
    source_markdown = """\\`code span`"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):\\\b`code span`:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>`code span`</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_1a():
    """
    Test case backslash extra 1a:  backslash before the code span closed
    """

    # Arrange
    source_markdown = """`code span\\`"""
    expected_tokens = [
        "[para(1,1):]",
        "[icode-span(1,1):code span\\:`::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><code>code span\\</code></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_2():
    """
    Test case backslash extra 2:  backslash before the character reference
    """

    # Arrange
    source_markdown = """\\&amp; the band played on"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):\\\b\a&\a&amp;\aamp; the band played on:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>&amp;amp; the band played on</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_3():
    """
    Test case backslash extra 3:  backslash before the inline html open
    """

    # Arrange
    source_markdown = """\\<there it='is'>, really"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):\\\b\a<\a&lt;\athere it='is'\a>\a&gt;\a, really:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>&lt;there it='is'&gt;, really</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_4():
    """
    Test case backslash extra 4:  backslash before the inline html close
    """

    # Arrange
    source_markdown = """<there it='is'\\>, really"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):\a<\a&lt;\athere it='is'\\\b\a>\a&gt;\a, really:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>&lt;there it='is'&gt;, really</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_5():
    """
    Test case backslash extra 5:  backslash before the autolink open
    """

    # Arrange
    source_markdown = """\\<http://www.google.com> is where to look"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):\\\b\a<\a&lt;\ahttp://www.google.com\a>\a&gt;\a is where to look:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>&lt;http://www.google.com&gt; is where to look</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_6():
    """
    Test case backslash extra 6:  backslash before the autolink close
    """

    # Arrange
    source_markdown = """<http://www.google.com\\> is where to look"""
    expected_tokens = [
        "[para(1,1):]",
        "[uri-autolink(1,1):http://www.google.com\\]",
        "[text(1,25): is where to look:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><a href="http://www.google.com%5C">http://www.google.com\\</a> is where to look</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_7():
    """
    Test case backslash extra 7:  backslash before the emphasis start
    """

    # Arrange
    source_markdown = """\\*it's* me!"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):\\\b*it's:]",
        "[text(1,7):*:]",
        "[text(1,8): me!:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>*it's* me!</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_8():
    """
    Test case backslash extra 8:  backslash before the emphasis end
    """

    # Arrange
    source_markdown = """*it's\\* me!"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):*:]",
        "[text(1,2):it's\\\b* me!:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>*it's* me!</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_9():
    """
    Test case backslash extra 9:  backslash before the first emphasis start
    """

    # Arrange
    source_markdown = """*\\*it's** me!"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):1:*]",
        "[text(1,2):\\\b*it's:]",
        "[end-emphasis(1,8)::]",
        "[text(1,9):*:]",
        "[text(1,10): me!:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><em>*it's</em>* me!</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_10():
    """
    Test case backslash extra 10:  backslash before the first emphasis end
    """

    # Arrange
    source_markdown = """**it's\\** me!"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):*:]",
        "[emphasis(1,2):1:*]",
        "[text(1,3):it's\\\b*:]",
        "[end-emphasis(1,9)::]",
        "[text(1,10): me!:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>*<em>it's*</em> me!</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_11():
    """
    Test case backslash extra 11:  backslash before the link open
    """

    # Arrange
    source_markdown = """\\[Foo](/uri) is a link"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):\\\b[Foo:]",
        "[text(1,6):]:]",
        "[text(1,7):(/uri) is a link:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>[Foo](/uri) is a link</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_12():
    """
    Test case backslash extra 12:  backslash before the image open
    """

    # Arrange
    source_markdown = """\\![foo](/url "title") is an image"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):\\\b!:]",
        '[link(1,3):inline:/url:title::::foo:False:":: :]',
        "[text(1,4):foo:]",
        "[end-link::]",
        "[text(1,22): is an image:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>!<a href="/url" title="title">foo</a> is an image</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_13():
    """
    Test case backslash extra 13:  backslash between the image open characters
    Also see: 600
    """

    # Arrange
    source_markdown = """!\\[foo](/url "title") is an image"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):!\\\b[foo:]",
        "[text(1,7):]:]",
        '[text(1,8):(/url \a"\a&quot;\atitle\a"\a&quot;\a) is an image:]',
        "[end-para:::True]",
    ]
    expected_gfm = """<p>![foo](/url &quot;title&quot;) is an image</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_14():
    """
    Test case backslash extra 14:  backslash before the code span open in setext
    """

    # Arrange
    source_markdown = """\\`code span`
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[text(1,1):\\\b`code span`:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2>`code span`</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_14a():
    """
    Test case backslash extra 14a:  backslash before the code span closed in setext
    """

    # Arrange
    source_markdown = """`code span\\`
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[icode-span(1,1):code span\\:`::]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2><code>code span\\</code></h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_15():
    """
    Test case backslash extra 15:  backslash before the character reference in setext
    """

    # Arrange
    source_markdown = """\\&amp; the band played on
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[text(1,1):\\\b\a&\a&amp;\aamp; the band played on:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2>&amp;amp; the band played on</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_16():
    """
    Test case backslash extra 16:  backslash before the inline html open in setext
    """

    # Arrange
    source_markdown = """\\<there it='is'>, really
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[text(1,1):\\\b\a<\a&lt;\athere it='is'\a>\a&gt;\a, really:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2>&lt;there it='is'&gt;, really</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_17():
    """
    Test case backslash extra 17:  backslash before the inline html close in setext
    """

    # Arrange
    source_markdown = """<there it='is'\\>, really
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[text(1,1):\a<\a&lt;\athere it='is'\\\b\a>\a&gt;\a, really:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2>&lt;there it='is'&gt;, really</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_18():
    """
    Test case backslash extra 18:  backslash before the autolink open in setext
    """

    # Arrange
    source_markdown = """\\<http://www.google.com> is where to look
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[text(1,1):\\\b\a<\a&lt;\ahttp://www.google.com\a>\a&gt;\a is where to look:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2>&lt;http://www.google.com&gt; is where to look</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_19():
    """
    Test case backslash extra 19:  backslash before the autolink close in setext
    """

    # Arrange
    source_markdown = """<http://www.google.com\\> is where to look
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[uri-autolink(1,1):http://www.google.com\\]",
        "[text(1,25): is where to look:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2><a href="http://www.google.com%5C">http://www.google.com\\</a> is where to look</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_20():
    """
    Test case backslash extra 20:  backslash before the emphasis start in setext
    """

    # Arrange
    source_markdown = """\\*it's* me!
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[text(1,1):\\\b*it's:]",
        "[text(1,7):*:]",
        "[text(1,8): me!:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2>*it's* me!</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_21():
    """
    Test case backslash extra 21:  backslash before the emphasis end in setext
    """

    # Arrange
    source_markdown = """*it's\\* me!
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[text(1,1):*:]",
        "[text(1,2):it's\\\b* me!:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2>*it's* me!</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_22():
    """
    Test case backslash extra 22:  backslash before the first emphasis start in setext
    """

    # Arrange
    source_markdown = """*\\*it's** me!
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[emphasis(1,1):1:*]",
        "[text(1,2):\\\b*it's:]",
        "[end-emphasis(1,8)::]",
        "[text(1,9):*:]",
        "[text(1,10): me!:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2><em>*it's</em>* me!</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_23():
    """
    Test case backslash extra 23:  backslash before the first emphasis end in setext
    """

    # Arrange
    source_markdown = """**it's\\** me!
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[text(1,1):*:]",
        "[emphasis(1,2):1:*]",
        "[text(1,3):it's\\\b*:]",
        "[end-emphasis(1,9)::]",
        "[text(1,10): me!:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2>*<em>it's*</em> me!</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_24():
    """
    Test case backslash extra 24:  backslash before the link open in setext
    """

    # Arrange
    source_markdown = """\\[Foo](/uri) is a link
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[text(1,1):\\\b[Foo:]",
        "[text(1,6):]:]",
        "[text(1,7):(/uri) is a link:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2>[Foo](/uri) is a link</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_25():
    """
    Test case backslash extra 25:  backslash before the image open in setext
    """

    # Arrange
    source_markdown = """\\![foo](/url "title") is an image
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[text(1,1):\\\b!:]",
        '[link(1,3):inline:/url:title::::foo:False:":: :]',
        "[text(1,4):foo:]",
        "[end-link::]",
        "[text(1,22): is an image:]",
        "[end-setext::]",
    ]
    expected_gfm = """<h2>!<a href="/url" title="title">foo</a> is an image</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_26():
    """
    Test case backslash extra 26:  backslash between the image open characters in setext
    """

    # Arrange
    source_markdown = """!\\[foo](/url "title") is an image
---"""
    expected_tokens = [
        "[setext(2,1):-:3::(1,1)]",
        "[text(1,1):!\\\b[foo:]",
        "[text(1,7):]:]",
        '[text(1,8):(/url \a"\a&quot;\atitle\a"\a&quot;\a) is an image:]',
        "[end-setext::]",
    ]
    expected_gfm = """<h2>![foo](/url &quot;title&quot;) is an image</h2>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_27():
    """
    Test case backslash extra 27:  backslash before the code span open in atx
    """

    # Arrange
    source_markdown = """# \\`code span`"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3):\\\b`code span`: ]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1>`code span`</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_27a():
    """
    Test case backslash extra 27a:  backslash before the code span closed in atx
    """

    # Arrange
    source_markdown = """# `code span\\`"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3)::\a \a\x03\a]",
        "[icode-span(1,3):code span\\:`::]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1><code>code span\\</code></h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_28():
    """
    Test case backslash extra 28:  backslash before the character reference in atx
    """

    # Arrange
    source_markdown = """# \\&amp; the band played on"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3):\\\b\a&\a&amp;\aamp; the band played on: ]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1>&amp;amp; the band played on</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_29():
    """
    Test case backslash extra 29:  backslash before the inline html open in atx
    """

    # Arrange
    source_markdown = """# \\<there it='is'>, really"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3):\\\b\a<\a&lt;\athere it='is'\a>\a&gt;\a, really: ]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1>&lt;there it='is'&gt;, really</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_30():
    """
    Test case backslash extra 30:  backslash before the inline html close in atx
    """

    # Arrange
    source_markdown = """# <there it='is'\\>, really"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3):\a<\a&lt;\athere it='is'\\\b\a>\a&gt;\a, really: ]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1>&lt;there it='is'&gt;, really</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_31():
    """
    Test case backslash extra 31:  backslash before the autolink open in atx
    """

    # Arrange
    source_markdown = """# \\<http://www.google.com> is where to look"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3):\\\b\a<\a&lt;\ahttp://www.google.com\a>\a&gt;\a is where to look: ]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1>&lt;http://www.google.com&gt; is where to look</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_32():
    """
    Test case backslash extra 32:  backslash before the autolink close in atx
    """

    # Arrange
    source_markdown = """# <http://www.google.com\\> is where to look"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3)::\a \a\x03\a]",
        "[uri-autolink(1,3):http://www.google.com\\]",
        "[text(1,27): is where to look:]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1><a href="http://www.google.com%5C">http://www.google.com\\</a> is where to look</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_33():
    """
    Test case backslash extra 33:  backslash before the emphasis start in atx
    """

    # Arrange
    source_markdown = """# \\*it's* me!"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3):\\\b*it's: ]",
        "[text(1,9):*:]",
        "[text(1,10): me!:]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1>*it's* me!</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_34():
    """
    Test case backslash extra 34:  backslash before the emphasis end in atx
    """

    # Arrange
    source_markdown = """# *it's\\* me!"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3)::\a \a\x03\a]",
        "[text(1,3):*:]",
        "[text(1,4):it's\\\b* me!:]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1>*it's* me!</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_35():
    """
    Test case backslash extra 35:  backslash before the first emphasis start in atx
    """

    # Arrange
    source_markdown = """# *\\*it's** me!"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3)::\a \a\x03\a]",
        "[emphasis(1,3):1:*]",
        "[text(1,4):\\\b*it's:]",
        "[end-emphasis(1,10)::]",
        "[text(1,11):*:]",
        "[text(1,12): me!:]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1><em>*it's</em>* me!</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_36():
    """
    Test case backslash extra 36:  backslash before the first emphasis end in atx
    """

    # Arrange
    source_markdown = """# **it's\\** me!"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3)::\a \a\x03\a]",
        "[text(1,3):*:]",
        "[emphasis(1,4):1:*]",
        "[text(1,5):it's\\\b*:]",
        "[end-emphasis(1,11)::]",
        "[text(1,12): me!:]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1>*<em>it's*</em> me!</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_37():
    """
    Test case backslash extra 37:  backslash before the link open in atx
    """

    # Arrange
    source_markdown = """# \\[Foo](/uri) is a link"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3):\\\b[Foo: ]",
        "[text(1,8):]:]",
        "[text(1,9):(/uri) is a link:]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1>[Foo](/uri) is a link</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_38():
    """
    Test case backslash extra 38:  backslash before the image open in atx
    """

    # Arrange
    source_markdown = """# \\![foo](/url "title") is an image"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3):\\\b!: ]",
        '[link(1,5):inline:/url:title::::foo:False:":: :]',
        "[text(1,6):foo:]",
        "[end-link::]",
        "[text(1,24): is an image:]",
        "[end-atx::]",
    ]
    expected_gfm = """<h1>!<a href="/url" title="title">foo</a> is an image</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_backslash_escapes_extra_39():
    """
    Test case backslash extra 39:  backslash between the image open characters in atx
    """

    # Arrange
    source_markdown = """# !\\[foo](/url "title") is an image"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        "[text(1,3):!\\\b[foo: ]",
        "[text(1,9):]:]",
        '[text(1,10):(/url \a"\a&quot;\atitle\a"\a&quot;\a) is an image:]',
        "[end-atx::]",
    ]
    expected_gfm = """<h1>![foo](/url &quot;title&quot;) is an image</h1>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)



"""
https://github.github.com/gfm/#html-blocks
"""
import pytest

from .utils import act_and_assert

# pylint: disable=too-many-lines


@pytest.mark.gfm
def test_html_blocks_extrax_01():
    """
    Test case 01:  Pragma alone in a document.
    """

    # Arrange
    source_markdown = """<!-- pyml -->"""
    expected_tokens = ["[pragma:1:<!-- pyml -->]"]
    expected_gfm = """"""

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )


@pytest.mark.gfm
def test_html_blocks_extrax_02():
    """
    Test case 02:  Pargma within a paragraph.
    """

    # Arrange
    source_markdown = """This is a paragraph
<!-- pyml -->
still a paragraph
<!-- pyml -->
and still going.
"""
    expected_tokens = [
        "[para(1,1):\n\n]",
        "[text(1,1):This is a paragraph\nstill a paragraph\nand still going.::\n\n]",
        "[end-para:::True]",
        "[BLANK(6,1):]",
        "[pragma:2:<!-- pyml -->;4:<!-- pyml -->]",
    ]
    expected_gfm = """<p>This is a paragraph\nstill a paragraph\nand still going.</p>"""

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )


@pytest.mark.gfm
def test_html_blocks_extrax_03():
    """
    Test case 03:  Pragma at the start and end of the document.
    """

    # Arrange
    source_markdown = """<!-- pyml -->
This is a paragraph
still a paragraph
and still going.
<!-- pyml -->"""
    expected_tokens = [
        "[para(2,1):\n\n]",
        "[text(2,1):This is a paragraph\nstill a paragraph\nand still going.::\n\n]",
        "[end-para:::True]",
        "[pragma:1:<!-- pyml -->;5:<!-- pyml -->]",
    ]
    expected_gfm = """<p>This is a paragraph\nstill a paragraph\nand still going.</p>"""

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )


@pytest.mark.gfm
def test_html_blocks_extrax_03a():
    """
    Test case 03a:  Pragma at start and end with a single line paragraph.
    """

    # Arrange
    source_markdown = """<!-- pyml -->
This is a paragraph.
<!-- pyml -->"""
    expected_tokens = [
        "[para(2,1):]",
        "[text(2,1):This is a paragraph.:]",
        "[end-para:::True]",
        "[pragma:1:<!-- pyml -->;3:<!-- pyml -->]",
    ]
    expected_gfm = """<p>This is a paragraph.</p>"""

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )


@pytest.mark.gfm
def test_html_blocks_extrax_04():
    """
    Test case 04:  Only two pragmas in entire document.
    """

    # Arrange
    source_markdown = """<!-- pyml -->
<!-- pyml -->"""
    expected_tokens = ["[pragma:1:<!-- pyml -->;2:<!-- pyml -->]"]
    expected_gfm = ""

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )


@pytest.mark.gfm
def test_html_blocks_extrax_05():
    """
    Test case 05:  Single line paragraph with double pragmas to start and end document.
    """

    # Arrange
    source_markdown = """<!-- pyml -->
<!-- pyml -->
this is a paragraph
<!-- pyml -->
<!-- pyml -->"""
    expected_tokens = [
        "[para(3,1):]",
        "[text(3,1):this is a paragraph:]",
        "[end-para:::True]",
        "[pragma:1:<!-- pyml -->;2:<!-- pyml -->;4:<!-- pyml -->;5:<!-- pyml -->]",
    ]
    expected_gfm = "<p>this is a paragraph</p>"

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )


@pytest.mark.gfm
def test_html_blocks_extrax_06():
    """
    Test case 06:  Verify that an HTML comment followed by the "pyml " title without any whitespace is parsed.
    """

    # Arrange
    source_markdown = """<!--pyml -->
this is a paragraph
"""
    expected_tokens = [
        "[para(2,1):]",
        "[text(2,1):this is a paragraph:]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[pragma:1:<!--pyml -->]",
    ]
    expected_gfm = "<p>this is a paragraph</p>"

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )


@pytest.mark.gfm
def test_html_blocks_extrax_07():
    """
    Test case 07:  Verify that an HTML comment followed by the "pyml " title with multiple whitespace is parsed.
    """

    # Arrange
    source_markdown = """<!-- \t \tpyml -->
this is a paragraph
"""
    expected_tokens = [
        "[para(2,1):]",
        "[text(2,1):this is a paragraph:]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[pragma:1:<!-- \t \tpyml -->]",
    ]
    expected_gfm = "<p>this is a paragraph</p>"

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )


@pytest.mark.gfm
def test_html_blocks_extrax_08():
    """
    Test case 08:  Pragma-like, without the space after the pragma title.
    """

    # Arrange
    source_markdown = """<!-- pyml-->
this is a paragraph
"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<!-- pyml-->:]",
        "[end-html-block:::False]",
        "[para(2,1):]",
        "[text(2,1):this is a paragraph:]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
    ]
    expected_gfm = "<!-- pyml-->\n<p>this is a paragraph</p>"

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )


@pytest.mark.gfm
def test_html_blocks_extrax_09():
    """
    Test case 08:  Pragma-like, without the closing comment sequence.
    """

    # Arrange
    source_markdown = """<!-- pyml--
this is a paragraph
"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<!-- pyml--\nthis is a paragraph:]",
        "[BLANK(3,1):]",
        "[end-html-block:::True]",
    ]
    expected_gfm = "<!-- pyml--\nthis is a paragraph\n"

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )


@pytest.mark.gfm
def test_html_blocks_extrax_010():
    """
    Test case 10:  Pragma heading, but with different casing.
    """

    # Arrange
    source_markdown = """<!-- PyML -->
this is a paragraph
"""
    expected_tokens = [
        "[para(2,1):]",
        "[text(2,1):this is a paragraph:]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[pragma:1:<!-- PyML -->]",
    ]
    expected_gfm = "<p>this is a paragraph</p>"

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )


@pytest.mark.gfm
def test_html_blocks_extrax_011():
    """
    Test case 11:  Pragma heading, but with extra spacing after the closing comment.
    """

    # Arrange
    source_markdown = """<!-- pyml -->\a\a\a
this is a paragraph
""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(2,1):]",
        "[text(2,1):this is a paragraph:]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[pragma:1:<!-- pyml -->   ]",
    ]
    expected_gfm = "<p>this is a paragraph</p>"

    # Act & Assert
    act_and_assert(
        source_markdown, expected_gfm, expected_tokens, disable_consistency_checks=True
    )



"""
https://github.github.com/gfm/#paragraph
"""
import pytest

from .utils import act_and_assert

# pylint: disable=too-many-lines


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_nl_fb():
    """
    Test case:  Ordered list newline fenced block
    was:        test_list_blocks_256jx
    """

    # Arrange
    source_markdown = """1.
```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[BLANK(1,3):]",
        "[end-olist:::True]",
        "[fcode-block(2,1):`:3::::::]",
        "[text(3,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<ol>
<li></li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_nl_all_i3_fb():
    """
    Test case:  Ordered list newline (all indented) fenced block
    """

    # Arrange
    source_markdown = """1.
   ```
   foo
   ```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3::   \n   \n   ]",
        "[BLANK(1,3):]",
        "[fcode-block(2,4):`:3::::::]",
        "[text(3,4):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(5,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<pre><code>foo
</code></pre>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_nl_i2_fb():
    """
    Test case:  Ordered list newline indent of 2 fenced block
    was:        test_list_blocks_256jxa
    """

    # Arrange
    source_markdown = """1.
  ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[BLANK(1,3):]",
        "[end-olist:::True]",
        "[fcode-block(2,3):`:3:::::  :]",
        "[text(3,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<ol>
<li></li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_nl_i3_fb():
    """
    Test case:  Ordered list newline indent of 2 fenced block
    was:        test_list_blocks_256jxb
    """

    # Arrange
    source_markdown = """1.
   ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3::   ]",
        "[BLANK(1,3):]",
        "[fcode-block(2,4):`:3::::::]",
        "[end-fcode-block:::True]",
        "[end-olist:::True]",
        "[para(3,1):]",
        "[text(3,1):foo:]",
        "[end-para:::False]",
        "[fcode-block(4,1):`:3::::::]",
        "[text(5,1)::]",
        "[end-fcode-block:::True]",
    ]
    expected_gfm = """<ol>
<li>
<pre><code></code></pre>
</li>
</ol>
<p>foo</p>
<pre><code></code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_t_nl_fb():
    """
    Test case:  Ordered list text newline fenced block
    was:        test_list_blocks_256ja
    """

    # Arrange
    source_markdown = """1.  abc
```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:4:]",
        "[para(1,5):]",
        "[text(1,5):abc:]",
        "[end-para:::True]",
        "[end-olist:::True]",
        "[fcode-block(2,1):`:3::::::]",
        "[text(3,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<ol>
<li>abc</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_t_nl_all_i4_fb():
    """
    Test case:  Ordered list text newline (all indented) fenced block
    """

    # Arrange
    source_markdown = """1.  abc
    ```
    foo
    ```
"""
    expected_tokens = [
        "[olist(1,1):.:1:4::    \n    \n    ]",
        "[para(1,5):]",
        "[text(1,5):abc:]",
        "[end-para:::False]",
        "[fcode-block(2,5):`:3::::::]",
        "[text(3,5):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(5,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
<pre><code>foo
</code></pre>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_t_nl_i2_fb():
    """
    Test case:  Ordered list text newline indent of 2 fenced block
    was:        test_list_blocks_256jaa
    """

    # Arrange
    source_markdown = """1.  abc
  ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:4:]",
        "[para(1,5):]",
        "[text(1,5):abc:]",
        "[end-para:::True]",
        "[end-olist:::True]",
        "[fcode-block(2,3):`:3:::::  :]",
        "[text(3,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<ol>
<li>abc</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_t_nl_i3_fb():
    """
    Test case:  Ordered list text newline indent of 3 fenced block
    was:        test_list_blocks_256jab
    """

    # Arrange
    source_markdown = """1.  abc
   ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:4:]",
        "[para(1,5):]",
        "[text(1,5):abc:]",
        "[end-para:::True]",
        "[end-olist:::True]",
        "[fcode-block(2,4):`:3:::::   :]",
        "[text(3,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<ol>
<li>abc</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_ol_nl_fb():
    """
    Test case:  Ordered list x2 newline fenced block
    """

    # Arrange
    source_markdown = """1. 1.
```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[olist(1,4):.:1:6:   ]",
        "[BLANK(1,6):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
        "[fcode-block(2,1):`:3::::::]",
        "[text(3,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li></li>
</ol>
</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_ol_nl_all_i6_fb():
    """
    Test case:  Ordered list x2 newline fenced block
    """

    # Arrange
    source_markdown = """1. 1.
      ```
      foo
      ```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[olist(1,4):.:1:6:   :      \n      \n      ]",
        "[BLANK(1,6):]",
        "[fcode-block(2,7):`:3::::::]",
        "[text(3,7):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(5,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>
<pre><code>foo
</code></pre>
</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_ol_t_nl_fb():
    """
    Test case:  Ordered list x2 text newline fenced block
    was:        test_list_blocks_256jb
    """

    # Arrange
    source_markdown = """1. 1. abc
```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[olist(1,4):.:1:6:   ]",
        "[para(1,7):]",
        "[text(1,7):abc:]",
        "[end-para:::True]",
        "[end-olist:::True]",
        "[end-olist:::True]",
        "[fcode-block(2,1):`:3::::::]",
        "[text(3,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>abc</li>
</ol>
</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_ol_t_nl_all_i6_fb():
    """
    Test case:  Ordered list x2 text newline fenced block
    """

    # Arrange
    source_markdown = """1. 1. abc
      ```
      foo
      ```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[olist(1,4):.:1:6:   :      \n      \n      ]",
        "[para(1,7):]",
        "[text(1,7):abc:]",
        "[end-para:::False]",
        "[fcode-block(2,7):`:3::::::]",
        "[text(3,7):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(5,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>abc
<pre><code>foo
</code></pre>
</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_nl_i3_ol_nl_fb():
    """
    Test case:  Ordered list newline indent of 3 ordered list newline fenced block
    """

    # Arrange
    source_markdown = """1.
   1.
```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[BLANK(1,3):]",
        "[olist(2,4):.:1:6:   ]",
        "[BLANK(2,6):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
        "[fcode-block(3,1):`:3::::::]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li></li>
</ol>
</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_t_nl_i3_ol_nl_fb():
    """
    Test case:  Ordered list text newline indent of 3 ordered list newline fenced block
    """

    # Arrange
    source_markdown = """1. abc
   1.
```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3::   ]",
        "[para(1,4):\n]",
        "[text(1,4):abc\n1.::\n]",
        "[end-para:::True]",
        "[end-olist:::True]",
        "[fcode-block(3,1):`:3::::::]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ol>
<li>abc
1.</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ul_t_nl_i2_ul_nl_fb():
    """
    Test case:  Unordered list text newline indent of 2 unordered list newline fenced block
    """

    # Arrange
    source_markdown = """- abc
  -
```
foo
```
"""
    expected_tokens = [
        "[ulist(1,1):-::2::  ]",
        "[setext(2,3):-:1::(1,3)]",
        "[text(1,3):abc:]",
        "[end-setext::]",
        "[end-ulist:::True]",
        "[fcode-block(3,1):`:3::::::]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ul>
<li>
<h2>abc</h2>
</li>
</ul>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ul_t_nl_i2_ulb_nl_fb():
    """
    Test case:  Unordered list text newline indent of 2 unordered list (b) newline fenced block
    """

    # Arrange
    source_markdown = """- abc
  *
```
foo
```
"""
    expected_tokens = [
        "[ulist(1,1):-::2::  ]",
        "[para(1,3):\n]",
        "[text(1,3):abc\n::\n]",
        "[text(2,1):*:]",
        "[end-para:::True]",
        "[end-ulist:::True]",
        "[fcode-block(3,1):`:3::::::]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ul>
<li>abc
*</li>
</ul>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_nl_i3_ol_t_nl_fb():
    """
    Test case:  Ordered list newline indent of 3 ordered list text newline fenced block
    was:        test_list_blocks_256jc
    """

    # Arrange
    source_markdown = """1.
   1. abc
```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[BLANK(1,3):]",
        "[olist(2,4):.:1:6:   ]",
        "[para(2,7):]",
        "[text(2,7):abc:]",
        "[end-para:::True]",
        "[end-olist:::True]",
        "[end-olist:::True]",
        "[fcode-block(3,1):`:3::::::]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>abc</li>
</ol>
</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_t_nl_i3_ol_t_nl_fb():
    """
    Test case:  Ordered list text newline indent of 3 ordered list text newline fenced block
    """

    # Arrange
    source_markdown = """1. abc
   1. abc
```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[para(1,4):]",
        "[text(1,4):abc:]",
        "[end-para:::True]",
        "[olist(2,4):.:1:6:   ]",
        "[para(2,7):]",
        "[text(2,7):abc:]",
        "[end-para:::True]",
        "[end-olist:::True]",
        "[end-olist:::True]",
        "[fcode-block(3,1):`:3::::::]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ol>
<li>abc
<ol>
<li>abc</li>
</ol>
</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_nl_i3_ol_nl_i2_fb():
    """
    Test case:  Ordered list newline indent of 3 ordered list newline indent of 2 fenced block
    """

    # Arrange
    source_markdown = """1.
   1.
  ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[BLANK(1,3):]",
        "[olist(2,4):.:1:6:   ]",
        "[BLANK(2,6):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
        "[fcode-block(3,3):`:3:::::  :]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li></li>
</ol>
</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_t_nl_i3_ol_nl_i2_fb():
    """
    Test case:  Ordered list text newline indent of 3 ordered list newline indent of 2 fenced block
    """

    # Arrange
    source_markdown = """1. abc
   1.
  ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3::   ]",
        "[para(1,4):\n]",
        "[text(1,4):abc\n1.::\n]",
        "[end-para:::True]",
        "[end-olist:::True]",
        "[fcode-block(3,3):`:3:::::  :]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ol>
<li>abc
1.</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ul_t_nl_i2_ul_nl_i1_fb():
    """
    Test case:  Unordered list text newline indent of 2 unordered list newline indent of 1 fenced block
    """

    # Arrange
    source_markdown = """- abc
  -
 ```
foo
```
"""
    expected_tokens = [
        "[ulist(1,1):-::2::  ]",
        "[setext(2,3):-:1::(1,3)]",
        "[text(1,3):abc:]",
        "[end-setext::]",
        "[end-ulist:::True]",
        "[fcode-block(3,2):`:3::::: :]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ul>
<li>
<h2>abc</h2>
</li>
</ul>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ul_t_nl_i2_ulb_nl_i1_fb():
    """
    Test case:  Unordered list text newline indent of 2 unordered list newline indent of 1 fenced block
    """

    # Arrange
    source_markdown = """- abc
  *
 ```
foo
```
"""
    expected_tokens = [
        "[ulist(1,1):-::2::  ]",
        "[para(1,3):\n]",
        "[text(1,3):abc\n::\n]",
        "[text(2,1):*:]",
        "[end-para:::True]",
        "[end-ulist:::True]",
        "[fcode-block(3,2):`:3::::: :]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ul>
<li>abc
*</li>
</ul>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_nl_i3_ol_t_nl_i2_fb():
    """
    Test case:  Ordered list newline indent of 3 ordered list text newline indent of 2 fenced block
    was:        test_list_blocks_256jd
    """

    # Arrange
    source_markdown = """1.
   1. abc
  ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[BLANK(1,3):]",
        "[olist(2,4):.:1:6:   ]",
        "[para(2,7):]",
        "[text(2,7):abc:]",
        "[end-para:::True]",
        "[end-olist:::True]",
        "[end-olist:::True]",
        "[fcode-block(3,3):`:3:::::  :]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>abc</li>
</ol>
</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_t_nl_i3_ol_t_nl_i2_fb():
    """
    Test case:  Ordered list text newline indent of 3 ordered list text newline indent of 2 fenced block
    """

    # Arrange
    source_markdown = """1. abc
   1. abc
  ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[para(1,4):]",
        "[text(1,4):abc:]",
        "[end-para:::True]",
        "[olist(2,4):.:1:6:   ]",
        "[para(2,7):]",
        "[text(2,7):abc:]",
        "[end-para:::True]",
        "[end-olist:::True]",
        "[end-olist:::True]",
        "[fcode-block(3,3):`:3:::::  :]",
        "[text(4,1):foo:]",
        "[end-fcode-block::3:False]",
        "[BLANK(6,1):]",
    ]
    expected_gfm = """<ol>
<li>abc
<ol>
<li>abc</li>
</ol>
</li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_nl_i3_ol_nl_i3_fb():
    """
    Test case:  Ordered list newline indent of 3 ordered list newline indent of 3 fenced block
    """

    # Arrange
    source_markdown = """1.
   1.
   ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3::   ]",
        "[BLANK(1,3):]",
        "[olist(2,4):.:1:6:   ]",
        "[BLANK(2,6):]",
        "[end-olist:::True]",
        "[fcode-block(3,4):`:3::::::]",
        "[end-fcode-block:::True]",
        "[end-olist:::True]",
        "[para(4,1):]",
        "[text(4,1):foo:]",
        "[end-para:::False]",
        "[fcode-block(5,1):`:3::::::]",
        "[text(6,1)::]",
        "[end-fcode-block:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li></li>
</ol>
<pre><code></code></pre>
</li>
</ol>
<p>foo</p>
<pre><code></code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_t_nl_i3_ol_nl_i3_fb():
    """
    Test case:  Ordered list text newline indent of 3 ordered list newline indent of 3 fenced block
    """

    # Arrange
    source_markdown = """1. abc
   1.
   ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3::   \n   ]",
        "[para(1,4):\n]",
        "[text(1,4):abc\n1.::\n]",
        "[end-para:::False]",
        "[fcode-block(3,4):`:3::::::]",
        "[end-fcode-block:::True]",
        "[end-olist:::True]",
        "[para(4,1):]",
        "[text(4,1):foo:]",
        "[end-para:::False]",
        "[fcode-block(5,1):`:3::::::]",
        "[text(6,1)::]",
        "[end-fcode-block:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
1.
<pre><code></code></pre>
</li>
</ol>
<p>foo</p>
<pre><code></code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ul_t_nl_i2_ul_nl_i2_fb():
    """
    Test case:  Unordered list text newline indent of 3 unordered list newline indent of 3 fenced block
    """

    # Arrange
    source_markdown = """- abc
  -
  ```
foo
```
"""
    expected_tokens = [
        "[ulist(1,1):-::2::  \n  ]",
        "[setext(2,3):-:1::(1,3)]",
        "[text(1,3):abc:]",
        "[end-setext::]",
        "[fcode-block(3,3):`:3::::::]",
        "[end-fcode-block:::True]",
        "[end-ulist:::True]",
        "[para(4,1):]",
        "[text(4,1):foo:]",
        "[end-para:::False]",
        "[fcode-block(5,1):`:3::::::]",
        "[text(6,1)::]",
        "[end-fcode-block:::True]",
    ]
    expected_gfm = """<ul>
<li>
<h2>abc</h2>
<pre><code></code></pre>
</li>
</ul>
<p>foo</p>
<pre><code></code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ul_t_nl_i2_ulb_nl_i2_fb():
    """
    Test case:  Unordered list text newline indent of 3 unordered list (b) newline indent of 3 fenced block
    """

    # Arrange
    source_markdown = """- abc
  *
  ```
foo
```
"""
    expected_tokens = [
        "[ulist(1,1):-::2::  \n  ]",
        "[para(1,3):\n]",
        "[text(1,3):abc\n::\n]",
        "[text(2,1):*:]",
        "[end-para:::False]",
        "[fcode-block(3,3):`:3::::::]",
        "[end-fcode-block:::True]",
        "[end-ulist:::True]",
        "[para(4,1):]",
        "[text(4,1):foo:]",
        "[end-para:::False]",
        "[fcode-block(5,1):`:3::::::]",
        "[text(6,1)::]",
        "[end-fcode-block:::True]",
    ]
    expected_gfm = """<ul>
<li>abc
*
<pre><code></code></pre>
</li>
</ul>
<p>foo</p>
<pre><code></code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_nl_i3_ol_t_nl_i3_fb():
    """
    Test case:  Ordered list newline indent of 3 ordered list text newline indent of 3 fenced block
    was:        test_list_blocks_256je
    """

    # Arrange
    source_markdown = """1.
   1. abc
   ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[BLANK(1,3):]",
        "[olist(2,4):.:1:6:   :   ]",
        "[para(2,7):]",
        "[text(2,7):abc:]",
        "[end-para:::False]",
        "[end-olist:::True]",
        "[fcode-block(3,4):`:3::::::]",
        "[end-fcode-block:::True]",
        "[end-olist:::True]",
        "[para(4,1):]",
        "[text(4,1):foo:]",
        "[end-para:::False]",
        "[fcode-block(5,1):`:3::::::]",
        "[text(6,1)::]",
        "[end-fcode-block:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>abc</li>
</ol>
<pre><code></code></pre>
</li>
</ol>
<p>foo</p>
<pre><code></code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_fb_ol_t_nl_i3_ol_t_nl_i3_fb():
    """
    Test case:  Ordered list text newline indent of 3 ordered list text newline indent of 3 fenced block
    """

    # Arrange
    source_markdown = """1. abc
   1. abc
   ```
foo
```
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[para(1,4):]",
        "[text(1,4):abc:]",
        "[end-para:::True]",
        "[olist(2,4):.:1:6:   :   ]",
        "[para(2,7):]",
        "[text(2,7):abc:]",
        "[end-para:::False]",
        "[end-olist:::True]",
        "[fcode-block(3,4):`:3::::::]",
        "[end-fcode-block:::True]",
        "[end-olist:::True]",
        "[para(4,1):]",
        "[text(4,1):foo:]",
        "[end-para:::False]",
        "[fcode-block(5,1):`:3::::::]",
        "[text(6,1)::]",
        "[end-fcode-block:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
<ol>
<li>abc</li>
</ol>
<pre><code></code></pre>
</li>
</ol>
<p>foo</p>
<pre><code></code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)



"""
Module to implement a plugin that looks for hard tabs in the files.
"""
from pymarkdown.plugin_manager import Plugin, PluginDetails


class RuleMd010(Plugin):
    """
    Class to implement a plugin that looks for hard tabs in the files.
    """

    def get_details(self):
        """
        Get the details for the plugin.
        """
        return PluginDetails(
            # whitespace, hard_tab
            plugin_name="no-hard-tabs",
            plugin_id="MD010",
            plugin_enabled_by_default=False,
            plugin_description="Hard tabs",
            plugin_version="0.0.0",
            plugin_interface_version=1,
        )  # https://github.com/DavidAnson/markdownlint/blob/master/doc/Rules.md#md010---hard-tabs
        # Parameters: code_blocks (boolean; default true)



"""
https://github.github.com/gfm/#html-blocks
"""
import pytest

from .utils import act_and_assert

# pylint: disable=too-many-lines


@pytest.mark.gfm
def test_html_blocks_118():
    """
    Test case 118:  (weird sample) <pre> within a HTML block started by <table> will not affect the parser state
    """

    # Arrange
    source_markdown = """<table><tr><td>
<pre>
**Hello**,

_world_.
</pre>
</td></tr></table>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<table><tr><td>\n<pre>\n**Hello**,:]",
        "[end-html-block:::False]",
        "[BLANK(4,1):]",
        "[para(5,1):\n]",
        "[emphasis(5,1):1:_]",
        "[text(5,2):world:]",
        "[end-emphasis(5,7)::]",
        "[text(5,8):.\n::\n]",
        "[raw-html(6,1):/pre]",
        "[end-para:::False]",
        "[html-block(7,1)]",
        "[text(7,1):</td></tr></table>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<table><tr><td>
<pre>
**Hello**,
<p><em>world</em>.
</pre></p>
</td></tr></table>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_119():
    """
    Test case 119:  (part 1) Some simple examples follow. Here are some basic HTML blocks of type 6:
    """

    # Arrange
    source_markdown = """<table>
  <tr>
    <td>
           hi
    </td>
  </tr>
</table>

okay."""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<table>\n  <tr>\n    <td>\n           hi\n    </td>\n  </tr>\n</table>:]",
        "[end-html-block:::False]",
        "[BLANK(8,1):]",
        "[para(9,1):]",
        "[text(9,1):okay.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<table>
  <tr>
    <td>
           hi
    </td>
  </tr>
</table>
<p>okay.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_120():
    """
    Test case 120:  (part 2) Some simple examples follow. Here are some basic HTML blocks of type 6:
    """

    # Arrange
    source_markdown = """ <div>
  *hello*
         <foo><a>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,2):<div>\n  *hello*\n         <foo><a>: ]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """ <div>
  *hello*
         <foo><a>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_121():
    """
    Test case 121:  A block can also start with a closing tag:
    """

    # Arrange
    source_markdown = """</div>
*foo*"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):</div>\n*foo*:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """</div>
*foo*"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_122():
    """
    Test case 122:  Here we have two HTML blocks with a Markdown paragraph between them:
    """

    # Arrange
    source_markdown = """<DIV CLASS="foo">

*Markdown*

</DIV>"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<DIV CLASS="foo">:]',
        "[end-html-block:::False]",
        "[BLANK(2,1):]",
        "[para(3,1):]",
        "[emphasis(3,1):1:*]",
        "[text(3,2):Markdown:]",
        "[end-emphasis(3,10)::]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[html-block(5,1)]",
        "[text(5,1):</DIV>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<DIV CLASS="foo">
<p><em>Markdown</em></p>
</DIV>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_123():
    """
    Test case 123:  (part 1) The tag on the first line can be partial, as long as it is split where there would be whitespace:
    """

    # Arrange
    source_markdown = """<div id="foo"
  class="bar">
</div>"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<div id="foo"\n  class="bar">\n</div>:]',
        "[end-html-block:::True]",
    ]
    expected_gfm = """<div id="foo"
  class="bar">
</div>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_123a():
    """
    Test case 123a:  variation
    """

    # Arrange
    source_markdown = """- <div id="foo"
class="bar">
</div>"""
    expected_tokens = [
        "[ulist(1,1):-::2:]",
        "[html-block(1,3)]",
        '[text(1,3):<div id="foo":]',
        "[end-html-block:::True]",
        "[end-ulist:::True]",
        "[para(2,1):]",
        '[text(2,1):class=\a"\a&quot;\abar\a"\a&quot;\a\a>\a&gt;\a:]',
        "[end-para:::False]",
        "[html-block(3,1)]",
        "[text(3,1):</div>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<ul>
<li>
<div id="foo"
</li>
</ul>
<p>class=&quot;bar&quot;&gt;</p>
</div>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_123bx():
    """
    Test case 123b:  variation
    """

    # Arrange
    source_markdown = """> <div id="foo"
class="bar">
</div>"""
    expected_tokens = [
        "[block-quote(1,1)::> ]",
        "[html-block(1,3)]",
        '[text(1,3):<div id="foo":]',
        "[end-html-block:::True]",
        "[end-block-quote:::True]",
        "[para(2,1):]",
        '[text(2,1):class=\a"\a&quot;\abar\a"\a&quot;\a\a>\a&gt;\a:]',
        "[end-para:::False]",
        "[html-block(3,1)]",
        "[text(3,1):</div>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<blockquote>
<div id="foo"
</blockquote>
<p>class=&quot;bar&quot;&gt;</p>
</div>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_123ba():
    """
    Test case 123ba:  variation
    """

    # Arrange
    source_markdown = """> <div id="foo"
> class="bar">
> </div>"""
    expected_tokens = [
        "[block-quote(1,1)::> \n> \n> ]",
        "[html-block(1,3)]",
        '[text(1,3):<div id="foo"\nclass="bar">\n</div>:]',
        "[end-html-block:::True]",
        "[end-block-quote:::True]",
    ]
    expected_gfm = """<blockquote>
<div id="foo"
class="bar">
</div>
</blockquote>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_123bb():
    """
    Test case 123bb:  variation
    """

    # Arrange
    source_markdown = """> <div id="foo"
>   class="bar">
> </div>"""
    expected_tokens = [
        "[block-quote(1,1)::> \n> \n> ]",
        "[html-block(1,3)]",
        '[text(1,3):<div id="foo"\n  class="bar">\n</div>:]',
        "[end-html-block:::True]",
        "[end-block-quote:::True]",
    ]
    expected_gfm = """<blockquote>
<div id="foo"
  class="bar">
</div>
</blockquote>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_123c():
    """
    Test case 123c:  variation
    """

    # Arrange
    source_markdown = """<div id="foo"
class="bar">
---"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<div id="foo"\nclass="bar">\n---:]',
        "[end-html-block:::True]",
    ]
    expected_gfm = """<div id="foo"
class="bar">
---"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_123d():
    """
    Test case 123d:  variation
    """

    # Arrange
    source_markdown = """# <div id="foo"
class="bar">"""
    expected_tokens = [
        "[atx(1,1):1:0:]",
        '[text(1,3):\a<\a&lt;\adiv id=\a"\a&quot;\afoo\a"\a&quot;\a: ]',
        "[end-atx::]",
        "[para(2,1):]",
        '[text(2,1):class=\a"\a&quot;\abar\a"\a&quot;\a\a>\a&gt;\a:]',
        "[end-para:::True]",
    ]
    expected_gfm = """<h1>&lt;div id=&quot;foo&quot;</h1>
<p>class=&quot;bar&quot;&gt;</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_124():
    """
    Test case 124:  (part 2) The tag on the first line can be partial, as long as it is split where there would be whitespace:
    """

    # Arrange
    source_markdown = """<div id="foo" class="bar
  baz">
</div>"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<div id="foo" class="bar\n  baz">\n</div>:]',
        "[end-html-block:::True]",
    ]
    expected_gfm = """<div id="foo" class="bar
  baz">
</div>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_125():
    """
    Test case 125:  An open tag need not be closed:
    """

    # Arrange
    source_markdown = """<div>
*foo*

*bar*"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<div>\n*foo*:]",
        "[end-html-block:::False]",
        "[BLANK(3,1):]",
        "[para(4,1):]",
        "[emphasis(4,1):1:*]",
        "[text(4,2):bar:]",
        "[end-emphasis(4,5)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<div>
*foo*
<p><em>bar</em></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_126():
    """
    Test case 126:  (part 1) A partial tag need not even be completed (garbage in, garbage out):
    """

    # Arrange
    source_markdown = """<div id="foo"
*hi*"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<div id="foo"\n*hi*:]',
        "[end-html-block:::True]",
    ]
    expected_gfm = """<div id="foo"
*hi*"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_127():
    """
    Test case 127:  (part 2) A partial tag need not even be completed (garbage in, garbage out):
    """

    # Arrange
    source_markdown = """<div class
foo"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<div class\nfoo:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<div class
foo"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_128():
    """
    Test case 128:  The initial tag doesn’t even need to be a valid tag, as long as it starts like one:
    """

    # Arrange
    source_markdown = """<div *???-&&&-<---
*foo*"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<div *???-&&&-<---\n*foo*:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<div *???-&&&-<---
*foo*"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_129():
    """
    Test case 129:  (part 1) In type 6 blocks, the initial tag need not be on a line by itself:
    """

    # Arrange
    source_markdown = """<div><a href="bar">*foo*</a></div>"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<div><a href="bar">*foo*</a></div>:]',
        "[end-html-block:::True]",
    ]
    expected_gfm = """<div><a href="bar">*foo*</a></div>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_130():
    """
    Test case 130:  (part 2) In type 6 blocks, the initial tag need not be on a line by itself:
    """

    # Arrange
    source_markdown = """<table><tr><td>
foo
</td></tr></table>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<table><tr><td>\nfoo\n</td></tr></table>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<table><tr><td>
foo
</td></tr></table>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_131():
    """
    Test case 131:  Everything until the next blank line or end of document gets included in the HTML block.
    """

    # Arrange
    source_markdown = """<div></div>
``` c
int x = 33;
```"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<div></div>\n``` c\nint x = 33;\n```:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<div></div>
``` c
int x = 33;
```"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_132():
    """
    Test case 132:  To start an HTML block with a tag that is not in the list of block-level tags in (6), you must put the tag by itself on the first line (and it must be complete):
    """

    # Arrange
    source_markdown = """<a href="foo">
*bar*
</a>"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<a href="foo">\n*bar*\n</a>:]',
        "[end-html-block:::True]",
    ]
    expected_gfm = """<a href="foo">
*bar*
</a>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_133():
    """
    Test case 133:  (part 1) In type 7 blocks, the tag name can be anything:
    """

    # Arrange
    source_markdown = """<Warning>
*bar*
</Warning>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<Warning>\n*bar*\n</Warning>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<Warning>
*bar*
</Warning>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_134():
    """
    Test case 134:  (part 2) In type 7 blocks, the tag name can be anything:
    """

    # Arrange
    source_markdown = """<i class="foo">
*bar*
</i>"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<i class="foo">\n*bar*\n</i>:]',
        "[end-html-block:::True]",
    ]
    expected_gfm = """<i class="foo">
*bar*
</i>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_135():
    """
    Test case 135:  (part 3) In type 7 blocks, the tag name can be anything:
    """

    # Arrange
    source_markdown = """</ins>
*bar*"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):</ins>\n*bar*:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """</ins>
*bar*"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_136():
    """
    Test case 136:  These rules are designed to allow us to work with tags that can function as either block-level or inline-level tags. The <del> tag is a nice example. We can surround content with <del> tags in three different ways. In this case, we get a raw HTML block, because the <del> tag is on a line by itself:
    """

    # Arrange
    source_markdown = """<del>
*foo*
</del>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<del>\n*foo*\n</del>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<del>
*foo*
</del>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_137():
    """
    Test case 137:  In this case, we get a raw HTML block that just includes the <del> tag (because it ends with the following blank line). So the contents get interpreted as CommonMark:
    """

    # Arrange
    source_markdown = """<del>

*foo*

</del>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<del>:]",
        "[end-html-block:::False]",
        "[BLANK(2,1):]",
        "[para(3,1):]",
        "[emphasis(3,1):1:*]",
        "[text(3,2):foo:]",
        "[end-emphasis(3,5)::]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[html-block(5,1)]",
        "[text(5,1):</del>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<del>
<p><em>foo</em></p>
</del>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_138():
    """
    Test case 138:  Finally, in this case, the <del> tags are interpreted as raw HTML inside the CommonMark paragraph. (Because the tag is not on a line by itself, we get inline HTML rather than an HTML block.)
    """

    # Arrange
    source_markdown = """<del>*foo*</del>"""
    expected_tokens = [
        "[para(1,1):]",
        "[raw-html(1,1):del]",
        "[emphasis(1,6):1:*]",
        "[text(1,7):foo:]",
        "[end-emphasis(1,10)::]",
        "[raw-html(1,11):/del]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><del><em>foo</em></del></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_139():
    """
    Test case 139:  A pre tag (type 1):
    """

    # Arrange
    source_markdown = """<pre language="haskell"><code>
import Text.HTML.TagSoup

main :: IO ()
main = print $ parseTags tags
</code></pre>
okay"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<pre language="haskell"><code>\nimport Text.HTML.TagSoup:]',
        "[BLANK(3,1):]",
        "[text(4,1):main :: IO ()\nmain = print $ parseTags tags\n</code></pre>:]",
        "[end-html-block:::False]",
        "[para(7,1):]",
        "[text(7,1):okay:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<pre language="haskell"><code>
import Text.HTML.TagSoup

main :: IO ()
main = print $ parseTags tags
</code></pre>
<p>okay</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_140():
    """
    Test case 140:  A script tag (type 1):
    """

    # Arrange
    source_markdown = """<script type="text/javascript">
// JavaScript example

document.getElementById("demo").innerHTML = "Hello JavaScript!";
</script>
okay"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<script type="text/javascript">\n// JavaScript example:]',
        "[BLANK(3,1):]",
        '[text(4,1):document.getElementById("demo").innerHTML = "Hello JavaScript!";\n</script>:]',
        "[end-html-block:::False]",
        "[para(6,1):]",
        "[text(6,1):okay:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<script type="text/javascript">
// JavaScript example

document.getElementById("demo").innerHTML = "Hello JavaScript!";
</script>
<p>okay</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_141():
    """
    Test case 141:  A style tag (type 1):
    """

    # Arrange
    source_markdown = """<style
  type="text/css">
h1 {color:red;}

p {color:blue;}
</style>
okay"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<style\n  type="text/css">\nh1 {color:red;}:]',
        "[BLANK(4,1):]",
        "[text(5,1):p {color:blue;}\n</style>:]",
        "[end-html-block:::False]",
        "[para(7,1):]",
        "[text(7,1):okay:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<style
  type="text/css">
h1 {color:red;}

p {color:blue;}
</style>
<p>okay</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_142():
    """
    Test case 142:  (part 1) If there is no matching end tag, the block will end at the end of the document (or the enclosing block quote or list item):
    """

    # Arrange
    source_markdown = """<style
  type="text/css">

foo"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<style\n  type="text/css">:]',
        "[BLANK(3,1):]",
        "[text(4,1):foo:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<style
  type="text/css">

foo"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_142a():
    """
    Test case 142:  (part 1) If there is no matching end tag, the block will end at the end of the document (or the enclosing block quote or list item):
    """

    # Arrange
    source_markdown = """<style
  foo="bar"
  type="text/css">

foo"""
    expected_tokens = [
        "[html-block(1,1)]",
        '[text(1,1):<style\n  foo="bar"\n  type="text/css">:]',
        "[BLANK(4,1):]",
        "[text(5,1):foo:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<style
  foo="bar"
  type="text/css">

foo"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_143x():
    """
    Test case 143:  (part 2) If there is no matching end tag, the block will end at the end of the document (or the enclosing block quote or list item):
    """

    # Arrange
    source_markdown = """> <div>
> foo

bar"""
    expected_tokens = [
        "[block-quote(1,1)::> \n> \n\n]",
        "[html-block(1,3)]",
        "[text(1,3):<div>\nfoo:]",
        "[end-html-block:::False]",
        "[BLANK(3,1):]",
        "[end-block-quote:::False]",
        "[para(4,1):]",
        "[text(4,1):bar:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<blockquote>
<div>
foo
</blockquote>
<p>bar</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_143a():
    """
    Test case 143a:  (part 2) If there is no matching end tag, the block will end at the end of the document (or the enclosing block quote or list item):
    """

    # Arrange
    source_markdown = """> <div>
> foo
> bar

bar"""
    expected_tokens = [
        "[block-quote(1,1)::> \n> \n> \n\n]",
        "[html-block(1,3)]",
        "[text(1,3):<div>\nfoo\nbar:]",
        "[end-html-block:::False]",
        "[BLANK(4,1):]",
        "[end-block-quote:::False]",
        "[para(5,1):]",
        "[text(5,1):bar:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<blockquote>
<div>
foo
bar
</blockquote>
<p>bar</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_143b():
    """
    Test case 143b:  (part 2) If there is no matching end tag, the block will end at the end of the document (or the enclosing block quote or list item):
    """

    # Arrange
    source_markdown = """> <div>
> foo
> bar
> baz

bar"""
    expected_tokens = [
        "[block-quote(1,1)::> \n> \n> \n> \n\n]",
        "[html-block(1,3)]",
        "[text(1,3):<div>\nfoo\nbar\nbaz:]",
        "[end-html-block:::False]",
        "[BLANK(5,1):]",
        "[end-block-quote:::False]",
        "[para(6,1):]",
        "[text(6,1):bar:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<blockquote>
<div>
foo
bar
baz
</blockquote>
<p>bar</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_143c():
    """
    Test case 143c:  (part 2) If there is no matching end tag, the block will end at the end of the document (or the enclosing block quote or list item):
    """

    # Arrange
    source_markdown = """> <div>
>foo
> bar
>baz

bar"""
    expected_tokens = [
        "[block-quote(1,1)::> \n>\n> \n>\n\n]",
        "[html-block(1,3)]",
        "[text(1,3):<div>\nfoo\nbar\nbaz:]",
        "[end-html-block:::False]",
        "[BLANK(5,1):]",
        "[end-block-quote:::False]",
        "[para(6,1):]",
        "[text(6,1):bar:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<blockquote>
<div>
foo
bar
baz
</blockquote>
<p>bar</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_144():
    """
    Test case 144:  (part 3) If there is no matching end tag, the block will end at the end of the document (or the enclosing block quote or list item):
    """

    # Arrange
    source_markdown = """- <div>
- foo"""
    expected_tokens = [
        "[ulist(1,1):-::2:]",
        "[html-block(1,3)]",
        "[text(1,3):<div>:]",
        "[end-html-block:::True]",
        "[li(2,1):2::]",
        "[para(2,3):]",
        "[text(2,3):foo:]",
        "[end-para:::True]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>
<div>
</li>
<li>foo</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_144a():
    """
    Test case 144a:  Modification of 144 to add extra paragraph
    """

    # Arrange
    source_markdown = """- <div>
- foo

foo"""
    expected_tokens = [
        "[ulist(1,1):-::2:]",
        "[html-block(1,3)]",
        "[text(1,3):<div>:]",
        "[end-html-block:::True]",
        "[li(2,1):2::]",
        "[para(2,3):]",
        "[text(2,3):foo:]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[end-ulist:::True]",
        "[para(4,1):]",
        "[text(4,1):foo:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<ul>
<li>
<div>
</li>
<li>foo</li>
</ul>
<p>foo</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_145():
    """
    Test case 145:  (part 1) The end tag can occur on the same line as the start tag:
    """

    # Arrange
    source_markdown = """<style>p{color:red;}</style>
*foo*"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<style>p{color:red;}</style>:]",
        "[end-html-block:::False]",
        "[para(2,1):]",
        "[emphasis(2,1):1:*]",
        "[text(2,2):foo:]",
        "[end-emphasis(2,5)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<style>p{color:red;}</style>
<p><em>foo</em></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_146():
    """
    Test case 146:  (part 2) The end tag can occur on the same line as the start tag:
    """

    # Arrange
    source_markdown = """<!-- foo -->*bar*
*baz*"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<!-- foo -->*bar*:]",
        "[end-html-block:::False]",
        "[para(2,1):]",
        "[emphasis(2,1):1:*]",
        "[text(2,2):baz:]",
        "[end-emphasis(2,5)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<!-- foo -->*bar*
<p><em>baz</em></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_147():
    """
    Test case 147:  Note that anything on the last line after the end tag will be included in the HTML block:
    """

    # Arrange
    source_markdown = """<script>
foo
</script>1. *bar*"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<script>\nfoo\n</script>1. *bar*:]",
        "[end-html-block:::False]",
    ]
    expected_gfm = """<script>
foo
</script>1. *bar*"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_148():
    """
    Test case 148:  A comment (type 2):
    """

    # Arrange
    source_markdown = """<!-- Foo

bar
   baz -->
okay"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<!-- Foo:]",
        "[BLANK(2,1):]",
        "[text(3,1):bar\n   baz -->:]",
        "[end-html-block:::False]",
        "[para(5,1):]",
        "[text(5,1):okay:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<!-- Foo

bar
   baz -->
<p>okay</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_149():
    """
    Test case 149:  A processing instruction (type 3):
    """

    # Arrange
    source_markdown = """<?php

  echo '>';

?>
okay"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<?php:]",
        "[BLANK(2,1):]",
        "[text(3,3):echo '>';:  ]",
        "[BLANK(4,1):]",
        "[text(5,1):?>:]",
        "[end-html-block:::False]",
        "[para(6,1):]",
        "[text(6,1):okay:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<?php

  echo '>';

?>
<p>okay</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_150():
    """
    Test case 150:  A declaration (type 4):
    """

    # Arrange
    source_markdown = """<!DOCTYPE html>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<!DOCTYPE html>:]",
        "[end-html-block:::False]",
    ]
    expected_gfm = """<!DOCTYPE html>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_151():
    """
    Test case 151:  CDATA (type 5):
    """

    # Arrange
    source_markdown = """<![CDATA[
function matchwo(a,b)
{
  if (a < b && a < 0) then {
    return 1;

  } else {

    return 0;
  }
}
]]>
okay"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<![CDATA[\nfunction matchwo(a,b)\n{\n  if (a < b && a < 0) then {\n    return 1;:]",
        "[BLANK(6,1):]",
        "[text(7,3):} else {:  ]",
        "[BLANK(8,1):]",
        "[text(9,5):return 0;\n  }\n}\n]]>:    ]",
        "[end-html-block:::False]",
        "[para(13,1):]",
        "[text(13,1):okay:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<![CDATA[
function matchwo(a,b)
{
  if (a < b && a < 0) then {
    return 1;

  } else {

    return 0;
  }
}
]]>
<p>okay</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_152():
    """
    Test case 152:  (part 1) The opening tag can be indented 1-3 spaces, but not 4:
    """

    # Arrange
    source_markdown = """  <!-- foo -->

    <!-- foo -->"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,3):<!-- foo -->:  ]",
        "[end-html-block:::False]",
        "[BLANK(2,1):]",
        "[icode-block(3,5):    :]",
        "[text(3,5):\a<\a&lt;\a!-- foo --\a>\a&gt;\a:]",
        "[end-icode-block:::True]",
    ]
    expected_gfm = """  <!-- foo -->
<pre><code>&lt;!-- foo --&gt;
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_153():
    """
    Test case 153:  (part 2) The opening tag can be indented 1-3 spaces, but not 4:
    """

    # Arrange
    source_markdown = """  <div>

    <div>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,3):<div>:  ]",
        "[end-html-block:::False]",
        "[BLANK(2,1):]",
        "[icode-block(3,5):    :]",
        "[text(3,5):\a<\a&lt;\adiv\a>\a&gt;\a:]",
        "[end-icode-block:::True]",
    ]
    expected_gfm = """  <div>
<pre><code>&lt;div&gt;
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_154():
    """
    Test case 154:  An HTML block of types 1–6 can interrupt a paragraph, and need not be preceded by a blank line.
    """

    # Arrange
    source_markdown = """Foo
<div>
bar
</div>"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):Foo:]",
        "[end-para:::False]",
        "[html-block(2,1)]",
        "[text(2,1):<div>\nbar\n</div>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<p>Foo</p>
<div>
bar
</div>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_155():
    """
    Test case 155:  However, a following blank line is needed, except at the end of a document, and except for blocks of types 1–5, above:
    """

    # Arrange
    source_markdown = """<div>
bar
</div>
*foo*"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<div>\nbar\n</div>\n*foo*:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<div>
bar
</div>
*foo*"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_156():
    """
    Test case 156:  HTML blocks of type 7 cannot interrupt a paragraph:
    """

    # Arrange
    source_markdown = """Foo
<a href="bar">
baz"""
    expected_tokens = [
        "[para(1,1):\n\n]",
        "[text(1,1):Foo\n::\n]",
        '[raw-html(2,1):a href="bar"]',
        "[text(2,15):\nbaz::\n]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>Foo
<a href="bar">
baz</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_157():
    """
    Test case 157:  (part 1) This rule differs from John Gruber’s original Markdown syntax specification
    """

    # Arrange
    source_markdown = """<div>

*Emphasized* text.

</div>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<div>:]",
        "[end-html-block:::False]",
        "[BLANK(2,1):]",
        "[para(3,1):]",
        "[emphasis(3,1):1:*]",
        "[text(3,2):Emphasized:]",
        "[end-emphasis(3,12)::]",
        "[text(3,13): text.:]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[html-block(5,1)]",
        "[text(5,1):</div>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<div>
<p><em>Emphasized</em> text.</p>
</div>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_158():
    """
    Test case 158:  (part 2) This rule differs from John Gruber’s original Markdown syntax specification
    """

    # Arrange
    source_markdown = """<div>
*Emphasized* text.
</div>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<div>\n*Emphasized* text.\n</div>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<div>
*Emphasized* text.
</div>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_159():
    """
    Test case 159:  The rule given above seems a simpler and more elegant way of achieving the same expressive power, which is also much simpler to parse.
    """

    # Arrange
    source_markdown = """<table>

<tr>

<td>
Hi
</td>

</tr>

</table>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<table>:]",
        "[end-html-block:::False]",
        "[BLANK(2,1):]",
        "[html-block(3,1)]",
        "[text(3,1):<tr>:]",
        "[end-html-block:::False]",
        "[BLANK(4,1):]",
        "[html-block(5,1)]",
        "[text(5,1):<td>\nHi\n</td>:]",
        "[end-html-block:::False]",
        "[BLANK(8,1):]",
        "[html-block(9,1)]",
        "[text(9,1):</tr>:]",
        "[end-html-block:::False]",
        "[BLANK(10,1):]",
        "[html-block(11,1)]",
        "[text(11,1):</table>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<table>
<tr>
<td>
Hi
</td>
</tr>
</table>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_160():
    """
    Test case 160:  The rule given above seems a simpler and more elegant way of achieving the same expressive power, which is also much simpler to parse.
    """

    # Arrange
    source_markdown = """<table>

  <tr>

    <td>
      Hi
    </td>

  </tr>

</table>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<table>:]",
        "[end-html-block:::False]",
        "[BLANK(2,1):]",
        "[html-block(3,1)]",
        "[text(3,3):<tr>:  ]",
        "[end-html-block:::False]",
        "[BLANK(4,1):]",
        "[icode-block(5,5):    :\n    \n    ]",
        "[text(5,5):\a<\a&lt;\atd\a>\a&gt;\a\n  Hi\n\a<\a&lt;\a/td\a>\a&gt;\a:]",
        "[end-icode-block:::False]",
        "[BLANK(8,1):]",
        "[html-block(9,1)]",
        "[text(9,3):</tr>:  ]",
        "[end-html-block:::False]",
        "[BLANK(10,1):]",
        "[html-block(11,1)]",
        "[text(11,1):</table>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<table>
  <tr>
<pre><code>&lt;td&gt;
  Hi
&lt;/td&gt;
</code></pre>
  </tr>
</table>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_160a():
    """
    Test case 160a:  Test case 160 with the blank lines removed
    """

    # Arrange
    source_markdown = """<table>
  <tr>
    <td>
      Hi
    </td>
  </tr>
</table>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<table>\n  <tr>\n    <td>\n      Hi\n    </td>\n  </tr>\n</table>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<table>
  <tr>
    <td>
      Hi
    </td>
  </tr>
</table>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_cov1():
    """
    Test case cov1:  Based on coverage analysis.
    """

    # Arrange
    source_markdown = """<hr/>
</x-table>

</x-table>"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<hr/>\n</x-table>:]",
        "[end-html-block:::False]",
        "[BLANK(3,1):]",
        "[html-block(4,1)]",
        "[text(4,1):</x-table>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<hr/>
</x-table>
</x-table>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_cov2x():
    """
    Test case cov2:  Based on coverage analysis.
    """

    # Arrange
    source_markdown = """</hrx
>
</x-table>"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):\a<\a&lt;\a/hrx:]",
        "[end-para:::True]",
        "[block-quote(2,1)::>]",
        "[BLANK(2,2):]",
        "[end-block-quote:::True]",
        "[html-block(3,1)]",
        "[text(3,1):</x-table>:]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<p>&lt;/hrx</p>
<blockquote>
</blockquote>
</x-table>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_cov2a():
    """
    Test case cov2:  Based on coverage analysis.
    """

    # Arrange
    source_markdown = """</hrx
>
> </x-table>"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):\a<\a&lt;\a/hrx:]",
        "[end-para:::True]",
        "[block-quote(2,1)::>\n> ]",
        "[BLANK(2,2):]",
        "[html-block(3,3)]",
        "[text(3,3):</x-table>:]",
        "[end-html-block:::True]",
        "[end-block-quote:::True]",
    ]
    expected_gfm = """<p>&lt;/hrx</p>
<blockquote>
</x-table>
</blockquote>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_cov2b():
    """
    Test case cov2:  Based on coverage analysis.
    """

    # Arrange
    source_markdown = """</hrx
>
 </x-table>"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):\a<\a&lt;\a/hrx:]",
        "[end-para:::True]",
        "[block-quote(2,1)::>]",
        "[BLANK(2,2):]",
        "[end-block-quote:::True]",
        "[html-block(3,1)]",
        "[text(3,2):</x-table>: ]",
        "[end-html-block:::True]",
    ]
    expected_gfm = """<p>&lt;/hrx</p>
<blockquote>
</blockquote>
 </x-table>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_cov3():
    """
    Test case cov3:  Based on coverage analysis.
    """

    # Arrange
    source_markdown = """<!bad>
</x-table>"""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):\a<\a&lt;\a!bad\a>\a&gt;\a\n::\n]",
        "[raw-html(2,1):/x-table]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>&lt;!bad&gt;
</x-table></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_cov4():
    """
    Test case cov4:  Based on coverage analysis.
    """

    # Arrange
    source_markdown = """<
bad>
</x-table>"""
    expected_tokens = [
        "[para(1,1):\n\n]",
        "[text(1,1):\a<\a&lt;\a\nbad\a>\a&gt;\a\n::\n\n]",
        "[raw-html(3,1):/x-table]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>&lt;
bad&gt;
</x-table></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_extra_01x():
    """
    Test case extra 01:  start a "list block" within a HTML code block
    """

    # Arrange
    source_markdown = """<script>
- some text
some other text
</script>
"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<script>\n- some text\nsome other text\n</script>:]",
        "[end-html-block:::False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<script>
- some text
some other text
</script>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_extra_01a():
    """
    Test case extra 01:  start a "list block" within a HTML code block
    """

    # Arrange
    source_markdown = """<script>
- [foo]:
/url
</script>
"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<script>\n- [foo]:\n/url\n</script>:]",
        "[end-html-block:::False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<script>
- [foo]:
/url
</script>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.skip
@pytest.mark.gfm
def test_html_blocks_extra_02x():
    """
    Test case extra 02:  variation of 1 where list already opened
    """

    # Arrange
    source_markdown = """- <script>
- some text
some other text
</script>
"""
    expected_tokens = [
        "[ulist(1,1):-::2::\n]",
        "[html-block(1,3)]",
        "[text(1,3):<script>:]",
        "[end-html-block:::True]",
        "[li(2,1):2::]",
        "[para(2,3):\n\n]",
        "[text(2,3):some text\nsome other text\n::\n\n]",
        "[raw-html(4,1):/script]",
        "[end-para:::True]",
        "[BLANK(5,1):]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>
<script>
</li>
<li>some text
some other text</li>
</ul>
</script>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.skip
@pytest.mark.gfm
def test_html_blocks_extra_02a():
    """
    Test case extra 02:  variation of 1 where list already opened
    """

    # Arrange
    source_markdown = """- <script>
- [foo]:
/url
</script>
"""
    expected_tokens = [
        "[ulist(1,1):-::2::\n]",
        "[html-block(1,3)]",
        "[text(1,3):<script>:]",
        "[link-ref-def(2,3):True::foo::\n:/url:::::]",
        "[end-ulist:::True]",
        "[html-block(4,1)]",
        "[text(4,1):</script>:]",
        "[end-html-block:::False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<ul>
<li>
<script>
</li>
<li></li>
</ul>
</script>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_extra_03x():
    """
    Test case extra 03:  variation of 1 where list already opened but no new list item
    """

    # Arrange
    source_markdown = """- <script>
  some text
some other text
</script>
"""
    expected_tokens = [
        "[ulist(1,1):-::2::  ]",
        "[html-block(1,3)]",
        "[text(1,3):<script>\nsome text:]",
        "[end-html-block:::True]",
        "[end-ulist:::True]",
        "[para(3,1):\n]",
        "[text(3,1):some other text\n::\n]",
        "[raw-html(4,1):/script]",
        "[end-para:::True]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<ul>
<li>
<script>
some text
</li>
</ul>
<p>some other text
</script></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.skip
@pytest.mark.gfm
def test_html_blocks_extra_03a():
    """
    Test case extra 03:  variation of 1 where list already opened but no new list item
    """

    # Arrange
    source_markdown = """- <script>
  [foo]:
/url
</script>
"""
    expected_tokens = [
        "[ulist(1,1):-::2::  \n\n]",
        "[html-block(1,3)]",
        "[text(1,3):<script>:]",
        "[link-ref-def(2,3):True::foo::\n:/url:::::]",
        "[text(4,1):</script>:]",
        "[end-html-block:::False]",
        "[end-ulist:::True]",
        "[html-block(4,1)]",
        "[text(4,1):</script>:]",
        "[end-html-block:::False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<ul>
<li>
<script>
[foo]:
</li>
</ul>
<p>/url
</script></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_extra_04x():
    """
    Test case extra 04:  variation of 3 where list already opened
    """

    # Arrange
    source_markdown = """<script>
> some text
some other text
</script>
"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<script>\n> some text\nsome other text\n</script>:]",
        "[end-html-block:::False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<script>
> some text
some other text
</script>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.skip
@pytest.mark.gfm
def test_html_blocks_extra_04a():
    """
    Test case extra 04:  variation of 3 where list already opened
    """

    # Arrange
    source_markdown = """<script>
> [foo]:
/url
</script>
"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<script>:]",
        "[end-html-block:::True]",
        "[block-quote(2,1)::> \n\n]",
        "[link-ref-def(2,3):True::foo::\n:/url:::::]",
        "[end-block-quote:::True]",
        "[html-block(4,1)]",
        "[text(4,1):</script>:]",
        "[end-html-block:::False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<script>
> [foo]:
/url
</script>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_extra_05x():
    """
    Test case extra 04:  variation of 3 where list already opened
    """

    # Arrange
    source_markdown = """> <script>
> some text
some other text
</script>
"""
    expected_tokens = [
        "[block-quote(1,1)::> \n> ]",
        "[html-block(1,3)]",
        "[text(1,3):<script>\nsome text:]",
        "[end-html-block:::True]",
        "[end-block-quote:::True]",
        "[para(3,1):\n]",
        "[text(3,1):some other text\n::\n]",
        "[raw-html(4,1):/script]",
        "[end-para:::True]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<blockquote>
<script>
some text
</blockquote>
<p>some other text
</script></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.skip
@pytest.mark.gfm
def test_html_blocks_extra_05a():
    """
    Test case extra 04:  variation of 3 where list already opened
    """

    # Arrange
    source_markdown = """> <script>
> [foo]:
/url
</script>
"""
    expected_tokens = [
        "[block-quote(1,1)::> \n> \n\n\n]",
        "[html-block(1,3)]",
        "[text(1,3):<script>:]",
        "[link-ref-def(2,3):True::foo::\n:/url:::::]",
        "[text(4,1):</script>:]",
        "[end-html-block:::False]",
        "[end-block-quote:::True]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<blockquote>
<script>
[foo]:
</blockquote>
<p>/url
</script></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_extra_06x():
    """
    Test case extra 05:  variation of 4 where list already opened but no new list item
    """

    # Arrange
    source_markdown = """> <script>
  some text
some other text
</script>
"""
    expected_tokens = [
        "[block-quote(1,1)::> ]",
        "[html-block(1,3)]",
        "[text(1,3):<script>:]",
        "[end-html-block:::True]",
        "[end-block-quote:::True]",
        "[para(2,3):  \n\n]",
        "[text(2,3):some text\nsome other text\n::\n\n]",
        "[raw-html(4,1):/script]",
        "[end-para:::True]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<blockquote>
<script>
</blockquote>
<p>some text
some other text
</script></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.skip
@pytest.mark.gfm
def test_html_blocks_extra_06a():
    """
    Test case extra 05:  variation of 4 where list already opened but no new list item
    """

    # Arrange
    source_markdown = """> <script>
  [foo]:
/url
</script>
"""
    expected_tokens = [
        "[block-quote(1,1)::> ]",
        "[html-block(1,3)]",
        "[text(1,3):<script>:]",
        "[link-ref-def(2,3):True:  :foo::\n:/url:::::]",
        "[html-block(4,1)]",
        "[text(4,1):</script>:]",
        "[end-html-block:::False]",
        "[BLANK(5,1):]",
    ]
    expected_gfm = """<blockquote>
<script>
</blockquote>
<p></script></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_html_blocks_extra_07():
    """
    Test case extra 07:  mixed block quotes and list blocks
    """

    # Arrange
    source_markdown = """<script>
* a
  > b
  >
* c
</script>
"""
    expected_tokens = [
        "[html-block(1,1)]",
        "[text(1,1):<script>\n* a\n  > b\n  >\n* c\n</script>:]",
        "[end-html-block:::False]",
        "[BLANK(7,1):]",
    ]
    expected_gfm = """<script>
* a
  > b
  >
* c
</script>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)



"""
Module to provide for...
"""
import logging

from pymarkdown.markdown_token import MarkdownToken, MarkdownTokenClass
from pymarkdown.parser_logger import ParserLogger

POGGER = ParserLogger(logging.getLogger(__name__))


class PragmaToken(MarkdownToken):
    """
    Token that contains the pragmas for the document.
    """

    pragma_prefix = "<!--"
    pragma_alternate_prefix = "<!---"
    pragma_title = "pyml "
    pragma_suffix = "-->"

    def __init__(self, pragma_lines):
        self.__pragma_lines = pragma_lines

        serialized_pragmas = ""
        for next_line_number in pragma_lines:
            serialized_pragmas += (
                ";" + str(next_line_number) + ":" + pragma_lines[next_line_number]
            )

        MarkdownToken.__init__(
            self,
            MarkdownToken._token_pragma,
            MarkdownTokenClass.SPECIAL,
            is_extension=True,
            extra_data=serialized_pragmas[1:],
        )

    @property
    def pragma_lines(self):
        """
        Returns the pragma lines for the document.
        """
        return self.__pragma_lines
